/******************************************************
 *
 * seq24: an hdf 24 bit images visualizer
 *        (only for gl-carded machines)
 *
 * Author: Andrea Leone (cirio@crs4.it)
 * Date: 6 July 1993
 *
 *****************************************************/

/* {{{ includes */

#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "hdf.h"

/* }}} */

/* {{{ prototypes */

int show(int num,
	 int32 xdim, 
	 int32 ydim, 
	 unsigned char *image);

void usage(char* prog);

int getImages(char *file,  
	      int32 *xdim, 
	      int32 *ydim, 
	      int *nImages,
	      unsigned char ***images);

int getnImages(char *file,  
	      int32 *xdim, 
	      int32 *ydim, 
	      int *nImages,
	      unsigned char ***images);

/* }}} */

int imageIndex;

main(int argc, char **argv) {

  int32 xdim, ydim, xaus, yaus;
  unsigned char *rr, *gg, *bb, **images;
  int nImages;
  
  char *file;
  
  int i;
  
  int menu;
  
  if ( (argc != 2) && (argc != 3))
    usage(argv[0]);
  
/* {{{ copying filename */

  file = (char*) malloc (strlen(argv[1])+1);
  if (file == NULL) {
	fprintf(stderr, "Cannot allocate memory!\n");
	exit(1);
      }

  strcpy(file,argv[1]);

/* }}} */

/* {{{ getting images */

  if (argc == 2) {
    if (getImages(file, &xdim, &ydim, &nImages, &images) == -1) {
      fprintf(stderr, "An error occurs while getting images\n");
      exit(1);
    }
  }
  else {
    nImages = atoi(argv[2]);
    if (nImages <= 0) {
      fprintf(stderr, "Incorrect number of images\n");
      exit(1);
    }
    if (getnImages(file, &xdim, &ydim, &nImages, &images) == -1) {
      fprintf(stderr, "An error occurs while getting images\n");
      exit(1);
    }
  }

/* }}} */

/* {{{ setting gl-window */

  prefsize(xdim, ydim);

  winopen("hdf24seq");
  wintitle("hdf24seq");

  RGBmode();
  singlebuffer();
  gconfig();

  clear();

  qdevice(RIGHTMOUSE); /* RightMouse */
  qdevice(REDRAW);     /* Redraw */
  qdevice(RKEY);       /* Reset key */
  qdevice(NKEY);       /* Next key */
  qdevice(PKEY);       /* Previous key */
  qdevice(SKEY);       /* Single run key */
  qdevice(LKEY);       /* Loop key */
  qdevice(QKEY);       /* Quit key */

/* }}} */

  imageIndex = 0;
  show(imageIndex, xdim, ydim, images[imageIndex]);

  menu = defpup("Main Menu%t|(R)eset|(N)ext|(P)revious|(S)ingle run|(L)oop|(Q)uit");

/* {{{ MAIN LOOP */

  for (;;) {
    short val;
#ifdef DEBUG
    fprintf(stderr, "Entering user-controlled loop\n");
#endif
    switch (qread(&val)) 
      {
      case REDRAW:
	winset(val);
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case RIGHTMOUSE:
	if (val) 
	  switch(dopup(menu))
	    {
	    case 1: /* Reset */
	      imageIndex=0;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 2: /* Next */
	      imageIndex = (imageIndex+1)%nImages;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 3: /* Previous */
	      imageIndex = (imageIndex+nImages-1)%nImages;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 4: /* Single run */
	      for (imageIndex = 0; imageIndex<nImages; imageIndex++)
		show(imageIndex, xdim, ydim, images[imageIndex]);
	      imageIndex--;
	      break;
	    case 5: /* Loop */
	      imageIndex = 0;
	      while(qtest()==0) {
		show(imageIndex, xdim, ydim, images[imageIndex]);
		imageIndex = (imageIndex+1)%nImages;
	      }
	      break;
	    case 6: /* Quit */
	      exit(1);
	    default:;
	    }
	break;
      case RKEY: /* Reset */
	qread(&val); /* to drop the keybord button release */
	imageIndex=0;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case NKEY: /* Next */
	qread(&val); /* to drop the keybord button release */
	imageIndex = (imageIndex+1)%nImages;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case PKEY: /* Previous */
	qread(&val); /* to drop the keybord button release */
	imageIndex = (imageIndex+nImages-1)%nImages;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case SKEY: /* Single run */
	qread(&val); /* to drop the keybord button release */
	for (imageIndex = 0; imageIndex<nImages; imageIndex++)
	  show(imageIndex, xdim, ydim, images[imageIndex]);
	imageIndex--;
	break;
      case LKEY: /* Loop */
	qread(&val); /* to drop the keybord button release */
	imageIndex = 0;
	while(qtest()==0) {
	  show(imageIndex, xdim, ydim, images[imageIndex]);
	  imageIndex = (imageIndex+1)%nImages;
	}
	break;
      case QKEY: /* Quit */
	exit(1);
      }
  }

/* }}} */

}


/* {{{ show */

int show(int num,
	 int32 xdim, 
	 int32 ydim, 
	 unsigned char *image) {

  unsigned char *rr, *gg, *bb;

  int row;
  
  fprintf(stderr, "Image No.%d\n", num);

#ifdef IBM6000
  lrectwrite(0,0,xdim-1,ydim-1, (Int32*)image);
#endif
#ifdef IRIS4
  lrectwrite(0,0,xdim-1,ydim-1, (unsigned long*)image);
#endif
  
}

/* }}} */

/* {{{ getImages */

int getImages(char *file,  
	      int32 *xdim, 
	      int32 *ydim, 
	      int *nImages,
	      unsigned char ***images) {

  int32 fid;
  int interlace;
  int i;
  int row, col, from, to;
  unsigned char *buffer;

/* {{{ getting the number of images in this file  */

/*
  fid = Hopen(file, DFACC_READ, 0);

  if (fid == -1) {
    fprintf(stderr, "An error occurs while opening the file\n");
    return(-1);
  }

  *nImages = Hnumber(fid, DFTAG_RI);

  Hclose(fid);
*/

  *nImages = DF24nimages(file);

  if (*nImages == -1) {
    fprintf(stderr, "An error occur while getting the number of images\n");
    return(-1);
  }

  if (*nImages == 0) {
    fprintf(stderr, "No 24-bit images in this file\n");
    return(-1);
  }

  printf("There are %d images in this file\n", *nImages);

/* }}} */

  fprintf(stderr, "Getting all images\n");

  *images = (unsigned char**) malloc (*nImages * sizeof (unsigned char*));
  if (*images == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }

  DF24restart();
  DF24reqil(0);
  if (DF24getdims(file, xdim, ydim, &interlace) != 0) {
    fprintf(stderr, "An error occur while getting image dimensions\n");
    return(-1);;
  }
  
  for (i=0; i<*nImages; i++) {
    ((unsigned char**)*images)[i] = (unsigned char*) malloc ( *xdim * *ydim * 4 );
    if (((unsigned char*)*images)[i] == NULL) {
      fprintf(stderr, "Cannot allocate memory\n");
      return(-1);
    }
  }
  
  buffer = (unsigned char*) malloc ( *xdim * *ydim * 3 );
  if (buffer == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }


  DF24restart();

  for (i=0; i<*nImages; i++) {
    
    printf("Getting image #%d\n", i);
    
    if (DF24getdims(file, xdim, ydim, &interlace) != 0) {
      fprintf(stderr, "An error occur while getting image dimensions\n");
      return(-1);
    }
    DF24reqil(0);
    if (DF24getimage(file, (char *)buffer, *xdim, *ydim) == -1) {
      fprintf(stderr, "An error occurs while getting images\n");
      return(-1);
    }
    for (row=0; row<*ydim; row++)
      for (col=0; col<*xdim; col++) {
	from = (*xdim*(*ydim-row-1)+col)*3;
	to   = (*xdim*row+col)*4;

/*
	(((char*)(*images)[i]))[to  ] = 0;
*/
        (((char*)(*images)[i]))[to+1] = buffer[from+2];
	(((char*)(*images)[i]))[to+2] = buffer[from+1];
	(((char*)(*images)[i]))[to+3] = buffer[from  ];
      }

  }
  
  return(0);
  
}

/* }}} */

/* {{{ getnImages */

int getnImages(char *file,  
	      int32 *xdim, 
	      int32 *ydim, 
	      int *nImages,
	      unsigned char ***images) {

  int32 fid;
  int interlace;
  int i;
  int row, col, from, to;
  unsigned char *buffer;
  int tmpnImages;

/* {{{ getting the number of images into the file  */

  fid = Hopen(file, DFACC_READ, 0);

  if (fid == -1) {
    fprintf(stderr, "An error occurs while opening the file\n");
    return(-1);
  }

  tmpnImages = Hnumber(fid, DFTAG_RI);

  Hclose(fid);

  if (tmpnImages == -1) {
    fprintf(stderr, "An error occur while getting the number of images\n");
    return(-1);
  }

  if (tmpnImages == 0) {
    fprintf(stderr, "No 24-bit images into the file\n");
    return(-1);
  }

  printf("There are %d images into the file\n", tmpnImages);

/* }}} */

  if (tmpnImages < *nImages) {
    *nImages = tmpnImages;
    fprintf(stderr, "Getting all images\n");
  }
  else {
    fprintf(stderr, "Getting only first %d images\n", *nImages);
  }
    
  *images = (unsigned char**) malloc (*nImages * sizeof (unsigned char*));
  if (*images == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }

  DF24restart();
  DF24reqil(0);

  if (DF24getdims(file, xdim, ydim, &interlace) != 0) {
    fprintf(stderr, "An error occur while getting image dimensions\n");
    return(-1);;
  }
  
  for (i=0; i<*nImages; i++) {
    ((unsigned char**)*images)[i] = (unsigned char*) malloc ( *xdim * *ydim * 4 );
    if (((unsigned char*)*images)[i] == NULL) {
      fprintf(stderr, "Cannot allocate memory\n");
      return(-1);
    }
  }
  
  buffer = (unsigned char*) malloc ( *xdim * *ydim * 3 );
  if (buffer == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }


  DF24restart();

  for (i=0; i<*nImages; i++) {
    
    printf("Getting image #%d\n", i);
    
    DF24reqil(0);
    if (DF24getimage(file, (char *)buffer, *xdim, *ydim) == -1) {
      fprintf(stderr, "An error occurs while getting images\n");
      return(-1);
    }
    for (row=0; row<*ydim; row++)
      for (col=0; col<*xdim; col++) {
	from = (*xdim*(*ydim-row-1)+col)*3;
	to   = (*xdim*row+col)*4;

/*
	(((char*)(*images)[i]))[to  ] = 0;
*/
        (((char*)(*images)[i]))[to+1] = buffer[from+2];
	(((char*)(*images)[i]))[to+2] = buffer[from+1];
	(((char*)(*images)[i]))[to+3] = buffer[from  ];
      }

  }
  
  return(0);
  
}

/* }}} */

/* {{{ usage */

void usage(char* prog) {
  fprintf(stderr, "Hdf 24 bit images sequencer\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "          Copyright 1993, CRS4\n");
  fprintf(stderr, "Centre for advanced Studies, R&D in Sardinia\n");
  fprintf(stderr, "          Via Nazario Sauro, 10\n");
  fprintf(stderr, "          09123, Cagliari, ITALY\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "usage: %s file [max images number]\n", prog);
  fprintf(stderr, "\n");
  exit(1);
}

/* }}} */

