/**********************************************************************
 *
 * multi-file hdf 24 bit images sequencer (October 1993)
 *
 * Created by: Andrea.Leone@crs4.it
 *
 **********************************************************************
 * 
 * 1993, Copyright CRS4
 * Centre for Advanced Studies, R&D in Sardinia
 * Via Nazario Sauro, 10, 09123 Cagliari (Italy)
 * tel +39 - 70 2796 282, fax +39 - 70 2796 283
 *
 **********************************************************************/

/* {{{ includes */

#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "hdf.h"

/* }}} */

/* {{{ prototypes */

int show(int num,
	 int32 xdim, 
	 int32 ydim, 
	 unsigned char *image);

void usage(char* prog);

int getImages(char *file,  
	      int nImages,
	      unsigned char *images);

int getImagesData(char *file, 
		  int *nImages, 
		  int32 *xdim, 
		  int32 *ydim);

/* }}} */

int imageIndex;

main(int argc, char **argv) {

  int32 xdim, ydim, xaus, yaus;
  unsigned char *rr, *gg, *bb, **images;
  int nImages;
  
  int i;
  
  typedef struct {
    int nImages;
    int32 xdim;
    int32 ydim;
  } imagesDataRec;

  imagesDataRec *imagesData;

  int menu;
  
  if (argc < 2)
    usage(argv[0]);
  
/* {{{ getting images data */

  /* allocating memory for images meta data */

  if ( (imagesData = (imagesDataRec*) malloc ( (argc-1) * sizeof(imagesDataRec) ) ) == NULL ) {
    fprintf(stderr, "Not enough memory\n");
    exit(1);
  }

  /* getting images meta data */

  nImages=0;
  for (i=0; i<argc-1; i++) {
    if (getImagesData(argv[i+1], 
		      &(imagesData[i].nImages),
		      &(imagesData[i].xdim),
		      &(imagesData[i].ydim)) == -1) {
      fprintf(stderr, 
	      "An error occur while getting images data from file %s\n", 
	      argv[i+1]);
      exit(1);
    }
#ifdef DEBUG
    fprintf(stderr, "DEBUG: file %d = %s (%d images)\n", i, argv[i+1], imagesData[i].nImages);
#endif
    nImages+=imagesData[i].nImages;
  }
   
  /* checking files image dimensions consistency */

  xdim = imagesData[0].xdim ;
  ydim = imagesData[0].ydim ;
  for (i=1; i<argc-1; i++)
    if ((imagesData[i].xdim != xdim) ||
	(imagesData[i].ydim != ydim)) {
      fprintf(stderr, 
	      "Dimension of images in file %s are different from that of file %s\n",
	      argv[i+1], argv[1]);
      exit(1);
    }
      

/* }}} */

/* {{{ getting images */

  /* allocating memory for all images */

  if ( (images = (unsigned char**) malloc (nImages * sizeof (unsigned char*))) == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }

  if ( (images[0] = (unsigned char*) malloc (nImages * 
					     imagesData[0].xdim * 
					     imagesData[0].ydim * 4)) == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }

  for (i=1; i<nImages; i++)
    images[i]=images[i-1]+(imagesData[0].xdim * imagesData[0].ydim * 4);
  
  /* getting images from files */

  nImages=0;

  for (i=0; i<argc-1; i++) {
    if (getImages(argv[i+1], imagesData[i].nImages, images[nImages]) == -1) {
      fprintf(stderr, "An error occurs while getting images from file %s\n", argv[i+1]);
      exit(1);
    }
    nImages += imagesData[i].nImages;
  }

/* }}} */

/* {{{ setting gl-window */

  prefsize(xdim, ydim);

  winopen("hdf24seq");
  wintitle("hdf24seq");

  RGBmode();
  singlebuffer();
  gconfig();

  clear();

  qdevice(RIGHTMOUSE); /* RightMouse */
  qdevice(REDRAW);     /* Redraw */
  qdevice(RKEY);       /* Reset key */
  qdevice(NKEY);       /* Next key */
  qdevice(PKEY);       /* Previous key */
  qdevice(SKEY);       /* Single run key */
  qdevice(LKEY);       /* Loop key */
  qdevice(QKEY);       /* Quit key */

/* }}} */

  imageIndex = 0;
  show(imageIndex, xdim, ydim, images[imageIndex]);

  menu = defpup("Main Menu%t|(R)eset|(N)ext|(P)revious|(S)ingle run|(L)oop|(Q)uit");

/* {{{ MAIN LOOP */

  for (;;) {
    short val;
#ifdef DEBUG
    fprintf(stderr, "Entering user-controlled loop\n");
#endif
    switch (qread(&val)) 
      {
      case REDRAW:
	winset(val);
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case RIGHTMOUSE:
	if (val) 
	  switch(dopup(menu))
	    {
	    case 1: /* Reset */
	      imageIndex=0;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 2: /* Next */
	      imageIndex = (imageIndex+1)%nImages;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 3: /* Previous */
	      imageIndex = (imageIndex+nImages-1)%nImages;
	      show(imageIndex, xdim, ydim, images[imageIndex]);
	      break;
	    case 4: /* Single run */
	      for (imageIndex = 0; imageIndex<nImages; imageIndex++)
		show(imageIndex, xdim, ydim, images[imageIndex]);
	      imageIndex--;
	      break;
	    case 5: /* Loop */
	      imageIndex = 0;
	      while(qtest()==0) {
		show(imageIndex, xdim, ydim, images[imageIndex]);
		imageIndex = (imageIndex+1)%nImages;
	      }
	      break;
	    case 6: /* Quit */
	      exit(1);
	    default:;
	    }
	break;
      case RKEY: /* Reset */
	qread(&val); /* to drop the keybord button release */
	imageIndex=0;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case NKEY: /* Next */
	qread(&val); /* to drop the keybord button release */
	imageIndex = (imageIndex+1)%nImages;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case PKEY: /* Previous */
	qread(&val); /* to drop the keybord button release */
	imageIndex = (imageIndex+nImages-1)%nImages;
	show(imageIndex, xdim, ydim, images[imageIndex]);
	break;
      case SKEY: /* Single run */
	qread(&val); /* to drop the keybord button release */
	for (imageIndex = 0; imageIndex<nImages; imageIndex++)
	  show(imageIndex, xdim, ydim, images[imageIndex]);
	imageIndex--;
	break;
      case LKEY: /* Loop */
	qread(&val); /* to drop the keybord button release */
	imageIndex = 0;
	while(qtest()==0) {
	  show(imageIndex, xdim, ydim, images[imageIndex]);
	  imageIndex = (imageIndex+1)%nImages;
	}
	break;
      case QKEY: /* Quit */
	exit(1);
      }
  }

/* }}} */

}


/* {{{ show */

int show(int num,
	 int32 xdim, 
	 int32 ydim, 
	 unsigned char *image) {

  unsigned char *rr, *gg, *bb;

  int row;
  
  fprintf(stderr, "Image No.%d\n", num);

#ifdef IBM6000
  lrectwrite(0,0,xdim-1,ydim-1, (Int32*)image);
#endif
#ifdef IRIS4
  lrectwrite(0,0,xdim-1,ydim-1, (unsigned long*)image);
#endif
  
}

/* }}} */

/* {{{ getImagesData */

int getImagesData(char *file, int *nImages, int32 *xdim, int32 *ydim) {

  int i;
  int interlace;
  int32 xaus, yaus;

/* getting the number of images in this file  */

  *nImages = DF24nimages(file);

  if (*nImages == -1) {
    fprintf(stderr, 
	    "An error occur while getting the number of images from the file %s\n", 
	    file);
    return(-1);
  }

  printf("There are %d images in file %s\n", *nImages, file);

  DF24restart();
  DF24reqil(0);
  if (DF24getdims(file, xdim, ydim, &interlace) != 0) {
    fprintf(stderr, "An error occur while getting image dimensions from file %s\n", file);
    HEprint(stderr, 0);
    return(-1);
  }
  xaus=*xdim;
  yaus=*ydim;
  
  for (i=1; i<*nImages; i++) {
    
    if (DF24getdims(file, xdim, ydim, &interlace) != 0) {
      fprintf(stderr, "An error occur while getting image dimensions\n");
      return(-1);
    }
    if ( (xaus != *xdim) || (yaus != *ydim) ) {
      fprintf(stderr, 
	      "Dimensions of image #%d in file %s are different from previous\n",
	      i, 
	      file);
      return(-1);
    }
  }

  return(0);

}

/* }}} */

/* {{{ getImages */

int getImages(char *file,  
	      int nImages,
	      unsigned char *images) {

  int32 fid;
  int32 xdim, ydim;
  int interlace;
  int i;
  int row, col, from, to;
  unsigned char *buffer;

  DF24restart();
  if (DF24getdims(file, &xdim, &ydim, &interlace) != 0) {
    fprintf(stderr, "An error occur while getting image dimensions from file %s\n", file);
    HEprint(stderr, 0);
    return(-1);
  }
  
  buffer = (unsigned char*) malloc ( xdim * ydim * 3 );
  if (buffer == NULL) {
    fprintf(stderr, "Cannot allocate memory\n");
    return(-1);
  }

  for (i=0; i<nImages; i++) {
    
    printf("Getting image #%d from file %s\n", i, file);
        
    DF24reqil(0);
    if (DF24getimage(file, (char *)buffer, xdim, ydim) == -1) {
      fprintf(stderr, "An error occurs while getting images\n");
      return(-1);
    }
    for (row=0; row<ydim; row++)
      for (col=0; col<xdim; col++) {
	from = (xdim*(ydim-row-1)+col)*3;
	to   = (xdim*row+col)*4;

/*
	((unsigned char*)images)[to  ] = 0;
*/
        ((unsigned char*)images)[to+1] = buffer[from+2];
	((unsigned char*)images)[to+2] = buffer[from+1];
	((unsigned char*)images)[to+3] = buffer[from  ];
      }
    images = images + (xdim*ydim*4);

  }
  
  free(buffer);

  return(0);
  
}

/* }}} */

/* {{{ usage */

void usage(char* prog) {
  fprintf(stderr, "Multi hdf files 24 bit images sequencer\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "          Copyright 1993, CRS4\n");
  fprintf(stderr, "Centre for advanced Studies, R&D in Sardinia\n");
  fprintf(stderr, "          Via Nazario Sauro, 10\n");
  fprintf(stderr, "          09123, Cagliari, ITALY\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "usage: %s file1 file2 ...\n", prog);
  fprintf(stderr, "\n");
  exit(1);
}

/* }}} */
